package sun.security.provider.certpath;

import sun.security.pkcs.SignerInfo;
import sun.security.pkcs.ContentInfo;
import sun.security.x509.AlgorithmId;
import java.util.ListIterator;
import sun.security.util.DerOutputStream;
import java.security.cert.CertificateEncodingException;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import sun.security.pkcs.PKCS7;
import sun.security.util.DerValue;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.security.cert.CertificateFactory;
import sun.security.util.DerInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.security.cert.CertificateException;
import java.security.cert.Certificate;
import java.util.Collection;
import java.security.cert.X509Certificate;
import java.util.List;
import java.security.cert.CertPath;

public class X509CertPath extends CertPath
{
    private static final long serialVersionUID = 4989800333263052980L;
    private List<X509Certificate> certs;
    private static final String COUNT_ENCODING = "count";
    private static final String PKCS7_ENCODING = "PKCS7";
    private static final String PKIPATH_ENCODING = "PkiPath";
    private static final Collection<String> encodingList;
    
    public X509CertPath(List<? extends Certificate> c) throws CertificateException {
        super("X.509");
        for (Object next : c) {
            if (!(next instanceof X509Certificate)) {
                throw new CertificateException("List is not all X509Certificates: " + next.getClass().getName());
            }
            continue;
        }
        this.certs = Collections.unmodifiableList(new ArrayList(c));
    }
    
    public X509CertPath(InputStream inputStream) throws CertificateException {
        this(inputStream, "PkiPath");
    }
    
    public X509CertPath(InputStream inputStream, String s) throws CertificateException {
        super("X.509");
        int n = -1;
        switch (s.hashCode()) {
            case 1148619507: {
                if (s.equals("PkiPath")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 76183020: {
                if (s.equals("PKCS7")) {
                    n = 1;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                this.certs = parsePKIPATH(inputStream);
                break;
            }
            case 1: {
                this.certs = parsePKCS7(inputStream);
                break;
            }
            default: {
                throw new CertificateException("unsupported encoding");
            }
        }
    }
    
    private static List<X509Certificate> parsePKIPATH(InputStream inputStream) throws CertificateException {
        if (inputStream == null) {
            throw new CertificateException("input stream is null");
        }
        try {
            DerValue[] sequence = new DerInputStream(readAllBytes(inputStream)).getSequence(3);
            if (sequence.length == 0) {
                return Collections.emptyList();
            }
            CertificateFactory instance = CertificateFactory.getInstance("X.509");
            ArrayList<Certificate> list = new ArrayList(sequence.length);
            for (int i = sequence.length - 1; i >= 0; --i) {
                list.add((Certificate)instance.generateCertificate(new ByteArrayInputStream(sequence[i].toByteArray())));
            }
            return Collections.unmodifiableList(list);
        }
        catch (IOException ex) {
            throw new CertificateException("IOException parsing PkiPath data: " + ex, ex);
        }
    }
    
    private static List<X509Certificate> parsePKCS7(InputStream in) throws CertificateException {
        if (in == null) {
            throw new CertificateException("input stream is null");
        }
        List list;
        try {
            if (!in.markSupported()) {
                in = new ByteArrayInputStream(readAllBytes(in));
            }
            X509Certificate[] certificates = new PKCS7(in).getCertificates();
            if (certificates != null) {
                list = Arrays.asList(certificates);
            }
            else {
                list = new ArrayList(0);
            }
        }
        catch (IOException obj) {
            throw new CertificateException("IOException parsing PKCS7 data: " + obj);
        }
        return Collections.unmodifiableList(list);
    }
    
    private static byte[] readAllBytes(InputStream inputStream) throws IOException {
        byte[] array = new byte[8192];
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(2048);
        int read;
        while ((read = inputStream.read(array)) != -1) {
            byteArrayOutputStream.write(array, 0, read);
        }
        return byteArrayOutputStream.toByteArray();
    }
    
    @Override
    public byte[] getEncoded() throws CertificateEncodingException {
        return this.encodePKIPATH();
    }
    
    private byte[] encodePKIPATH() throws CertificateEncodingException {
        ListIterator<X509Certificate> listIterator = this.certs.listIterator(this.certs.size());
        try {
            DerOutputStream derOutputStream = new DerOutputStream();
            while (listIterator.hasPrevious()) {
                X509Certificate x509Certificate = (X509Certificate)listIterator.previous();
                if (this.certs.lastIndexOf(x509Certificate) != this.certs.indexOf(x509Certificate)) {
                    throw new CertificateEncodingException("Duplicate Certificate");
                }
                derOutputStream.write(x509Certificate.getEncoded());
            }
            DerOutputStream o = new DerOutputStream();
            o.write((byte)48, derOutputStream);
            return o.toByteArray();
        }
        catch (IOException ex) {
            throw new CertificateEncodingException("IOException encoding PkiPath data: " + ex, ex);
        }
    }
    
    private byte[] encodePKCS7() throws CertificateEncodingException {
        PKCS7 pkcs7 = new PKCS7(new AlgorithmId[0], new ContentInfo(ContentInfo.DATA_OID, null), (X509Certificate[])this.certs.toArray(new X509Certificate[this.certs.size()]), new SignerInfo[0]);
        DerOutputStream out = new DerOutputStream();
        try {
            pkcs7.encodeSignedData(out);
        }
        catch (IOException ex) {
            throw new CertificateEncodingException(ex.getMessage());
        }
        return out.toByteArray();
    }
    
    @Override
    public byte[] getEncoded(String s) throws CertificateEncodingException {
        int n = -1;
        switch (s.hashCode()) {
            case 1148619507: {
                if (s.equals("PkiPath")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 76183020: {
                if (s.equals("PKCS7")) {
                    n = 1;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                return this.encodePKIPATH();
            }
            case 1: {
                return this.encodePKCS7();
            }
            default: {
                throw new CertificateEncodingException("unsupported encoding");
            }
        }
    }
    
    public static Iterator<String> getEncodingsStatic() {
        return X509CertPath.encodingList.iterator();
    }
    
    @Override
    public Iterator<String> getEncodings() {
        return getEncodingsStatic();
    }
    
    @Override
    public List<X509Certificate> getCertificates() {
        return this.certs;
    }
    
    static {
        ArrayList c = new ArrayList(2);
        c.add("PkiPath");
        c.add("PKCS7");
        encodingList = Collections.unmodifiableCollection(c);
    }
}
